/************************************************************************
	main.c

    RC Servo Switcher
    Copyright (C) 2011 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef MAIN_C
#define MAIN_C

// Global includes
#include <htc.h>

// Fuse configuration for the PIC12F683
__CONFIG(FCMDIS & IESODIS & BORDIS & UNPROTECT & MCLREN &
	PWRTDIS & WDTDIS & INTIO);

// Define oscillator frequency
#define _XTAL_FREQ 8000000

// Hardware mapping definitions
#define PULSE_IN	GPIO2
#define RC_SELECT	GPIO5
#define MC_SELECT	GPIO4

// Useful definitions
#define OFF		0
#define ON		1
#define FALSE	0
#define TRUE	1

// Main procedure
void main(void)
{
	OSCCON = 0b01110001;	// 8Mhz INT OSC
	
	// Set up the PIC12F683 IO pins
	GPIO   = 0b00000000;	// Set all pins to zero
	ANSEL  = 0b00000000;	// Disable analogue inputs
	TRISIO = 0b00000100;	// Set all IO pins to output
	OPTION = 0b00000000;	// Set the option register (page 12 of DS)
	WPU    = 0b00000000;	// Set weak pull-up off on all pins
	CMCON0 = 7;				// Disable comparator 0
	
	// Default to radio control
	MC_SELECT = 0;
	RC_SELECT = 1;
	
	// Configure timer1 for use with the Capture/Compare module
	
	// Fosc is 8 Mhz so Fosc/4 is 2,000,000 timer ticks per second.
	// We need to measure 10-30 milliseconds durations which is
	// 10,000-20,000 microseconds or 20,000 to 40,000 timer ticks.
	//
	// Timer1 is 16-bit, however we will use a 1:2 prescaler since the
	// timer value will then be (conveniently) microseconds.
	//
	TMR1IF = 0; // Reset the timer1 interrupt flag
	T1SYNC = 0;
	T1CON = 0b00010001;
	TMR1IE = 1; // Turn on the timer1 interrupt
	TMR1H = 0x00;
	TMR1L = 0x00;
	
	// Configure the Capture/Compare module
	CCP1CON = 0b00000101; // Capture mode every rising edge of GPIO2
	CCP1IE = 0; // CCP interrupt disabled
	CCP1IF = 0; // Clear the capture flag
	
	unsigned int currentPulseTime = 0;
	
	unsigned int validPulseCount = 0;
	unsigned int pulseCounter = 0;
	
	unsigned char radioDetected = FALSE;
	
	while(1)
	{
		if (TMR1IF == 1) // Timer has overflowed
		{
			// Overflow is at 65535 microseconds
			
			// Reset the timer
			TMR1H = 0x00;
			TMR1L = 0x00;
			TMR1IF = 0;
			
			pulseCounter++;
		}
		
		if (CCP1IF == 1) // CCP module interrupt flag set
		{
			// Stop the timer
			TMR1ON = 0;
			
			// Read and convert the timer1 value
			currentPulseTime = TMR1H << 8;
			currentPulseTime += TMR1L;
			
			// Reset the timer
			TMR1H = 0x00;
			TMR1L = 0x00;
			
			// Restart the timer
			TMR1IF = 0;
			TMR1ON = 1;
			
			// Reset the CCP
			CCP1IF = 0;
		
			// Do we have a valid pulse time?
			if (currentPulseTime > 10000 && currentPulseTime < 30000)
				validPulseCount++;
				
			pulseCounter++;
		}
		
		if (pulseCounter == 40)
		{
			// We must have 30 valid pulses per 40 from the RC to consider it 'on'
			if (validPulseCount < 30) radioDetected = FALSE;
			else radioDetected = TRUE;
			
			validPulseCount = 0;
			pulseCounter = 0;
		}		

		// Select the correct servo controlling device
		if (radioDetected == TRUE)
		{
			// Select servo control from the radio receiver
			MC_SELECT = 0;
			RC_SELECT = 1;
		}
		else
		{
			// Select servo control from the microcontroller
			RC_SELECT = 0;
			MC_SELECT = 1;
		}		
	}	
}

#endif